//: Playground - noun: a place where people can play

import Foundation

enum AppVersionRelation {
    case LessThan
    case EqualTo
    case GreaterThan
}

/// Compares two dot-separated application version strings.
func compare(appVersion lhs: String, with rhs: String) -> AppVersionRelation {
    return compare(appVersion: lhs, with: rhs, separator: ".")
}

/// Compares two application version strings, with a specified separator.
func compare(appVersion lhs: String, with rhs: String, separator: String = ".") -> AppVersionRelation {
    guard lhs != rhs else { return .EqualTo }

    let separator: Character = "." // Version ordinals are dot-separated
    let lhsCharacters = lhs.characters.split(separator: separator).map { String($0) }
    let rhsCharacters = rhs.characters.split(separator: separator).map { String($0) }

    let lhsCount = lhsCharacters.count
    let rhsCount = rhsCharacters.count

    // Set index, e.g., `i`, to first inequal ordinal or length of shortest version string:
    var i = 0
    while i < lhsCount && i < rhsCount && lhsCharacters[i] == rhsCharacters[i] {
        i += 1
    }

    guard i < lhsCount && i < rhsCount else {
        if lhsCount > rhsCount && lhsCharacters[i] == "0" {
            return .EqualTo
        } else if lhsCount < rhsCount && rhsCharacters[i] == "0" {
            return .EqualTo
        } else if lhsCount < rhsCount {
            return .LessThan
        } else if lhsCount > rhsCount {
            return .GreaterThan
        } else {
            return .EqualTo
        }
    }

    let lhsInt = Int(lhsCharacters[i]) ?? 0
    let rhsInt = Int(rhsCharacters[i]) ?? 0

    // comparison with failing ternary
    func compare1(integer lhs: Int, with rhs: Int) -> AppVersionRelation {
        guard lhs != rhs else { return .EqualTo }
        // return lhs < rhs ? .LessThan : .GreaterTham // <-- failing operator `<`
        return .LessThan // comment this to reproduce
    }

    // identical comparison with working ternary
    func compare2(integer lhs: Int, with rhs: Int) -> AppVersionRelation {
        guard lhs != rhs else { return .EqualTo }
        return lhs < rhs ? .LessThan : .GreaterThan
    }

    return compare2(integer: lhsInt, with: rhsInt)
}

// MARK: - Tests

let version1 = "4.6"
let version2 = "4.6.0"
let version3 = "4.6.1"
let version4 = "4.7"
let version5 = "4.7.2"

compare(appVersion: version1, with: version1) // 4.6 == 4.6
compare(appVersion: version1, with: version2) // 4.6 == 4.6.0
compare(appVersion: version1, with: version3) // 4.6 < 4.6.1
compare(appVersion: version1, with: version4) // 4.6 < 4.7
compare(appVersion: version1, with: version5) // 4.6 < 4.7.2
compare(appVersion: version5, with: version1) // 4.7.2 > 4.6
